# -*- coding: UTF-8 -*-

from django.conf.urls.defaults import *

urlpatterns = patterns('basic.views',
    url(r'^replace/$', 'replace', name="replace"),
    url(r'^hide/$', 'hide', name="hide"),
    url(r'^messages/$', 'messages', name="messages"),
    url(r'^update/$', 'update',  name="update"),
    url(r'^forms/$', 'forms',  name="forms"),
    url(r'^redirect/$', 'redirect',  name="redirect"),
    url(r'^django_redirect/$', 'django_redirect',  name="django_redirect"),
    url(r'^extra/$', 'extra',  name="extra"),
    url(r'^extra_2/$', 'extra_2',  name="extra_2"),
    url(r'^template_include/tag/$', 'template_include_tag',  name="template_include_tag"),
    url(r'^template_include/update/$', 'template_include_update',  name="template_include_update"),
    url(r'^do_everything/$', 'do_everything',  name="do_everything"),
    url(r'^do_nothing/$', 'do_nothing',  name="do_nothing"),
    url(r'^$', 'demo',  name="demo"),
)




""" Adjax testing views.

    The following views make use of the adjax library, allowing them to be tested.
"""

import adjax

@adjax.adjax_response
def replace(request):
    """ Replace an HTML element """
    adjax.replace(request, '#abc', 'Hello world')
   
@adjax.adjax_response
def hide(request):
    """ Hide an HTML element """
    adjax.hide(request, '#xyz')

@adjax.adjax_response
def messages(request):
    """ Display some messages """
    adjax.success(request, "This is your first success")
    adjax.info(request, "This is your first info")
    adjax.warning(request, "This is your first warning")
    adjax.error(request, "This is your first error")
    adjax.debug(request, "This is your first debug")
   
@adjax.adjax_response
def update(request):
    """ Update a django object """
    my_obj = MyModel(name="Abc", color="blue", price=123)
    adjax.update(request, my_obj, ('name', 'color'))

from django import forms
from basic.models import MyModel
class MyForm(forms.ModelForm):
    class Meta:
        model = MyModel
   
@adjax.adjax_response
def forms(request):
    """ Display form validation errors. """
    if request.method == "POST":
        my_form = MyForm(request.POST, prefix="withprefix")
        if my_form.is_valid():
            adjax.redirect(request, 'do_nothing')
    else:
        my_form = MyForm({'name': "Blah", 'price': "123"}, prefix="withprefix") # color missing!
    adjax.form(request, my_form)
   
@adjax.adjax_response
def redirect(request):
    """ Redirect the browser (using javascript). """
    adjax.redirect(request, 'do_nothing')
   
@adjax.adjax_response
def django_redirect(request):
    """ This makes use of the standard django redirect function. """
    from django.shortcuts import redirect
    return redirect('do_nothing')

@adjax.adjax_response
def extra(request):
    """ Add information to the json response """
    return {'one': 123}

@adjax.adjax_response
def extra_2(request):
    """ Add extra information to the ajax response explicitly """
    adjax.extra(request, 'two', 234)
    return {'one': 123}

@adjax.adjax_response
def template_include_update(request):
    """ Replace rendered content with the output of a template. """
    adjax.render_to_response(request, 'basic/_included.html', {'abc': 'xyz123'})
    adjax.render_to_response(request, 'basic/_included.html', {'abc': 'mno456'}, prefix="tree")

def template_include_tag(request):
    """ Render a template that uses the template include tag. """
    from django.shortcuts import render_to_response
    return render_to_response('basic/template_includer.html', {'abc': '13579'})

@adjax.adjax_response
def do_everything(request):
    """ Putting everything together. """
    adjax.replace(request, '#abc', 'Hello world')
    adjax.hide(request, '#xyz')
    adjax.success(request, "This is your first success")
    adjax.info(request, "This is your first info")
    adjax.warning(request, "This is your first warning")
    adjax.error(request, "This is your first error")
    adjax.debug(request, "This is your first debug")
    my_obj = MyModel(name="Abc", color="blue", price=123)
    adjax.update(request, my_obj, ('name', 'color'))
    my_form = MyForm({'name': "ABC", 'price': 123}, prefix="withprefix")
    adjax.form(request, my_form)
    adjax.redirect(request, 'do_nothing')
    adjax.extra(request, 'two', 234)
    adjax.render_to_response(request, 'basic/_included.html', {'abc': 'xyz123'})
    adjax.render_to_response(request, 'basic/_included.html', {'abc': 'mno456'}, prefix="tree")
    return {'one': 123}

def do_nothing(request):
    """ A view that can't go wrong! 
        This is a good target for testing redirects.
    """
    from django.http import HttpResponse
    return HttpResponse()

def demo(request):
    """ This view provides a demonstration page for Adjax.
    """
    context = {}
    context['form'] = MyForm(prefix="withprefix")
    context['my_obj'] = MyModel(name="Tree", color="green", price=899)
    context['abc'] = "123 waiting 456"
    from django.shortcuts import render_to_response
    from django.template.context import RequestContext
    return render_to_response('basic/demo.html', context, context_instance=RequestContext(request))



# 
# Adjax, a framework for easing the development of Django websites with Ajax.
#

__all__ = ('adjax_response', 'success', 'info', 'warning', 'error', 'debug', 
           'redirect', 'update', 'form', 'replace', 'hide', 'extra',   
           'render_to_response')

__version_info__ = ('1', '0', '0')
__version__ = '.'.join(__version_info__)
__authors__ = ["Will Hardy <adjax@hardysoftware.com.au>"]


from adjax.decorators import adjax_response
from adjax.base import get_store
from django.contrib.messages import success, info, warning, error, debug


def update(request, obj, attributes=None):
    """ Sends the updated version of the given attributes on the given object. 
        If no attributes are given, all attributes are sent (be careful if you
        don't want all data to be public).
        If a minus sign is in front of an attribute, it is omitted.
        A mix of attribtue names with and without minus signs is just silly.
        No other attributes will be included.
    """
    store = get_store(request)
    if not attributes or all(map(lambda s: s.startswith("-"), attributes)):
        attributes = obj.__dict__.keys()
    store.update(obj, (a for a in attributes if not a.startswith("-")))


def form(request, form_obj):
    """ Validate the given form and send errors to browser. """
    get_store(request).form(form_obj)


def replace(request, element, html):
    """ Replace the given DOM element with the given html. 
        The DOM element is specified using css identifiers.
        Some javascript libraries may have an extended syntax, 
        which can be used if you don't value portability.
    """
    get_store(request).replace(element, html)


def redirect(request, path):
    """ Redirect the browser dynamically to another page. """
    get_store(request).redirect(path)


def hide(request, element):
    """ Hides the given DOM element.
        The DOM element is specified using css identifiers.
        Some javascript libraries may have an extended syntax, 
        which can be used if you don't value portability.
    """
    get_store(request).hide(element)


def extra(request, key, value):
    """ Send additional information to the browser. """
    get_store(request).extra(key, value)


def render_to_response(request, template_name, context=None, prefix=None):
    """ Update any included templates. """
    get_store(request).render_to_response(template_name, context, prefix)



# -*- coding: UTF-8 -*-

__all__ = ('adjax_response',)

from django.http import HttpResponse
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext
from django.conf import settings

from django.core.serializers import json, serialize

from adjax.base import get_store
from django.utils.functional import Promise 
from django.utils.encoding import force_unicode 

class LazyEncoder(json.DjangoJSONEncoder): 
    def default(self, obj): 
        if isinstance(obj, Promise): 
            return force_unicode(obj) 
        return super(LazyEncoder, self).default(obj)

class JsonResponse(HttpResponse):
    def __init__(self, object):
        if isinstance(object, QuerySet):
            content = serialize('json', object)
        else:
            content = simplejson.dumps(
                object, indent=2, cls=LazyEncoder,
                ensure_ascii=False)
        super(JsonResponse, self).__init__(
            content, content_type='application/json')

# Where to redirect to when view is called without an ajax request.
DEFAULT_REDIRECT = getattr(settings, 'ADJAX_DEFAULT_REDIRECT', None)
ADJAX_CONTEXT_KEY = 'adjax'


def adjax_response(func):
    """ Renders the response using JSON, if appropriate.
    """
    # TODO allow a template to be given for non-ajax requests
    template_name = None

    def wrapper(request, *args, **kw):
        output = func(request, *args, **kw)
        store = get_store(request)

        # If a dict is given, add that to the output
        if output is None:
            output = {}
        elif isinstance(output, dict):
            output = output.copy()
            output.pop('request', None)
            for key, val in output.items():
                store.extra(key, val)

        # Intercept redirects
        elif isinstance(output, HttpResponse) and output.status_code in (301, 302):
            store.redirect(output['Location'])

        if request.is_ajax():
            return store.json_response

        if isinstance(output, dict):
            # If we have a template, render that
            if template_name:
                output.setdefault(ADJAX_CONTEXT_KEY, store)
                return render_to_response(template_name, output, context_instance=RequestContext(request))

            # Try and redirect somewhere useful
            if 'HTTP_REFERER' in request.META:
                return redirect(request.META['HTTP_REFERER'])
            elif DEFAULT_REDIRECT:
                return redirect(DEFAULT_REDIRECT)
            else:
                return HttpResponse()

        return output

    return wrapper



# -*- coding: UTF-8 -*-

from django.core.serializers import json, serialize
from django.http import HttpResponse
from django.utils import simplejson
from django.db.models.query import QuerySet

try:
    import hashlib
    hash_function = hashlib.sha1
except ImportError:
    import sha
    hash_function = sha.new


def get_key(instance, field_name):
    """ Returns the key that will be used to identify dynamic fields in the DOM. """
    # TODO: Avoid any characters that may not appear in class names
    m = instance._meta
    return '-'.join(('data', m.app_label, m.object_name, str(instance.pk), field_name))


def get_template_include_key(template_name, prefix=None):
    """ Get a valid element class name, we'll stick to ascii letters, numbers and hyphens.
        NB class names cannot start with a hyphen
    """
    digest = int(hash_function(template_name).hexdigest(),16)
    hash = base36.from_decimal(digest)
    if prefix:
        return 'tpl-%s-%s' % (prefix, hash)
    else:
        return 'tpl-%s' % (hash)


class JsonResponse(HttpResponse):
    def __init__(self, obj):
        if isinstance(obj, QuerySet):
            content = serialize('json', obj)
        else:
            content = simplejson.dumps(obj, indent=2, cls=json.DjangoJSONEncoder, ensure_ascii=False)

        super(JsonResponse, self).__init__(content, content_type='application/json')


"""
Convert numbers from base 10 integers to base X strings and back again.

Sample usage:

>>> base20 = BaseConverter('0123456789abcdefghij')
>>> base20.from_decimal(1234)
'31e'
>>> base20.to_decimal('31e')
1234

From http://www.djangosnippets.org/snippets/1431/

"""

class BaseConverter(object):
    decimal_digits = "0123456789"
    
    def __init__(self, digits):
        self.digits = digits
    
    def from_decimal(self, i):
        return self.convert(i, self.decimal_digits, self.digits)
    
    def to_decimal(self, s):
        return int(self.convert(s, self.digits, self.decimal_digits))
    
    def convert(number, fromdigits, todigits):
        # Based on http://code.activestate.com/recipes/111286/
        if str(number)[0] == '-':
            number = str(number)[1:]
            neg = 1
        else:
            neg = 0

        # make an integer out of the number
        x = 0
        for digit in str(number):
           x = x * len(fromdigits) + fromdigits.index(digit)
    
        # create the result in base 'len(todigits)'
        if x == 0:
            res = todigits[0]
        else:
            res = ""
            while x > 0:
                digit = x % len(todigits)
                res = todigits[digit] + res
                x = int(x / len(todigits))
            if neg:
                res = '-' + res
        return res
    convert = staticmethod(convert)

base36 = BaseConverter('0123456789abcdefghijklmnopqrstuvwxyz')



from django.db import models

class MyModel(models.Model):
    name  = models.CharField(max_length=25)
    color = models.CharField(max_length=20)
    price = models.IntegerField(default=100)

    def __unicode__(self):
        return self.name



"""
This file demonstrates two different styles of tests (one doctest and one
unittest). These will both pass when you run "manage.py test".

Replace these with more appropriate tests for your application.
"""

from django.test import TestCase, Client
from django.core.urlresolvers import reverse
from django.utils import simplejson

from adjax.utils import get_key, get_template_include_key
from basic.models import MyModel

class BasicTests(TestCase):
    def get_view(self, name):
        self.client = Client()
        response = self.client.get(reverse(name), HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        return  simplejson.loads(response.content)

    def assert_value_in(self, container, value, *keys):
        for key in keys:
            try:
                container = container[key]
            except (IndexError, KeyError):
                self.fail('"%s" missing from container: %s' % (key, repr(container)))
        self.assertEqual(container, value)

    def test_replace(self):
        self.assert_value_in(self.get_view('replace'), 'Hello world', 'replace', '#abc')

    def test_hide(self):
        self.assert_value_in(self.get_view('hide'), '#xyz', 'hide', 0)

    def test_messages(self):
        data = self.get_view('messages')
        assert 'messages' in data, repr(data)
        for message in data['messages']:
            assert message['content'].startswith(u"This is your first "), message
            assert message['content'].endswith(message['tags']), message

    def test_update(self):
        data = self.get_view('update')
        assert 'update' in data, repr(data)
        model = MyModel(name="Abc", color="blue")
        name_key = get_key(model, 'name')
        color_key = get_key(model, 'color')
        price_key = get_key(model, 'price')
        update_dict = data['update']
        assert price_key not in update_dict
        try:
            self.assertEqual(update_dict[name_key], "Abc")
            self.assertEqual(update_dict[color_key], "blue")
        except KeyError:
            self.fail(repr(update_dict))

    def test_forms(self):
        data = self.get_view('forms')
        assert 'forms' in data, repr(data)
        assert 'id_withprefix-color' in data['forms'], repr(data)
        self.assertEqual(data['forms']['id_withprefix-color'][0], u"This field is required.")

    def test_redirect(self):
        data = self.get_view('redirect')
        assert 'redirect' in data, repr(data)
        self.assertEqual(data['redirect'], reverse('do_nothing'))

    def test_django_redirect(self):
        data = self.get_view('django_redirect')
        assert 'redirect' in data, repr(data)
        self.assertEqual(data['redirect'], reverse('do_nothing'))

    def test_extra(self):
        data = self.get_view('extra')
        assert 'extra' in data, repr(data)
        assert 'one' in data['extra'], repr(data['extra'])
        self.assertEqual(data['extra']['one'], 123)

    def test_extra_2(self):
        data = self.get_view('extra_2')
        assert 'extra' in data, repr(data)
        assert 'one' in data['extra'], repr(data['extra'])
        self.assertEqual(data['extra']['one'], 123)
        assert 'two' in data['extra'], repr(data['extra'])
        self.assertEqual(data['extra']['two'], 234)

    def test_template_include_update(self):
        """ Check the correct information is sent to the browser when updating
            a template include. 
        """
        data = self.get_view('template_include_update')
        assert 'replace' in data, repr(data)
        key = ".%s" % get_template_include_key('basic/_included.html')
        assert key in data['replace'], repr(data['replace'])
        assert 'xyz123' in data['replace'][key]
        key = ".%s" % get_template_include_key('basic/_included.html', prefix="tree")
        assert key in data['replace'], repr(data['replace'])
        assert 'mno456' in data['replace'][key]

    def test_template_include_tag_1(self):
        """ Checks that the template include tag correctly includes the given
            templates.
        """
        response = self.client.get(reverse('template_include_tag'))
        key = get_template_include_key('basic/_included.html')
        self.assertContains(response, "13579")
        self.assertContains(response, '<div class="%s"' % key)

    def test_template_include_tag_2(self):
        """ Checks that the template include tag can use a given element type.
        """
        response = self.client.get(reverse('template_include_tag'))
        key = get_template_include_key('basic/_included.html')
        self.assertContains(response, '<p class="%s"' % key)

    def test_template_include_tag_3(self):
        """ Checks that the template include tag can have a prefix.
        """
        response = self.client.get(reverse('template_include_tag'))
        key = get_template_include_key('basic/_included.html', prefix="tree")
        self.assertContains(response, '<div class="%s"' % key)

    def test_do_everything(self):
        """ A quick check on the ability to use all functionality at once. """
        data = self.get_view('do_everything')
        assert 'replace' in data, repr(data)
        assert 'hide' in data, repr(data)
        assert 'messages' in data, repr(data)
        assert 'update' in data, repr(data)
        assert 'forms' in data, repr(data)
        assert 'redirect' in data, repr(data)
        assert 'extra' in data, repr(data)

        # replace
        self.assert_value_in(data, 'Hello world', 'replace', '#abc')
        # hide
        self.assert_value_in(data, '#xyz', 'hide', 0)
        # messages
        for message in data['messages']:
            assert message['content'].startswith(u"This is your first "), message
            assert message['content'].endswith(message['tags']), message
        # update
        model = MyModel(name="Abc", color="blue")
        name_key = get_key(model, 'name')
        color_key = get_key(model, 'color')
        price_key = get_key(model, 'price')
        update_dict = data['update']
        assert price_key not in update_dict
        try:
            self.assertEqual(update_dict[name_key], "Abc")
            self.assertEqual(update_dict[color_key], "blue")
        except KeyError:
            self.fail(repr(update_dict))
        # forms
        assert 'id_withprefix-color' in data['forms'], repr(data)
        self.assertEqual(data['forms']['id_withprefix-color'][0], u"This field is required.")
        # redirect
        self.assertEqual(data['redirect'], reverse('do_nothing'))
        # extra
        self.assertEqual(data['extra']['one'], 123)
        self.assertEqual(data['extra']['two'], 234)
        # template include
        key = ".%s" % get_template_include_key('basic/_included.html')
        assert key in data['replace'], repr(data['replace'])
        assert 'xyz123' in data['replace'][key]
        key = ".%s" % get_template_include_key('basic/_included.html', prefix="tree")
        assert key in data['replace'], repr(data['replace'])
        assert 'mno456' in data['replace'][key]



# -*- coding: UTF-8 -*-

from utils import get_key, JsonResponse, get_template_include_key
from django.contrib import messages
from django.core import urlresolvers
from django.template.context import RequestContext
from django.template.loader import render_to_string


def get_store(request):
    """ Gets a relevant store object from the given request. """
    if not hasattr(request, '_adjax_store'):
        request._adjax_store = AdjaxStore(request)
    return request._adjax_store


class AdjaxStore(object):
    """ This class will help store ajax data collected in views. """

    def __init__(self, request):
        self.request = request
        self.update_data = {}
        self.form_data = {}
        self.replace_data = {}
        self.hide_data = []
        self.extra_data = {}
        self.redirect_data = None

    @property
    def messages_data(self):
        return [{'tags': m.tags, 'content': unicode(m), 'level': m.level} for m in messages.get_messages(self.request)]

    def update(self, obj, attributes=None):
        """ Make values from a given object available. """
        for attr in attributes:
            value = getattr(obj, attr)
            if callable(value):
                value = value()
            self.update_data[get_key(obj, attr)] = value

    def form(self, form_obj):
        """ Validate the given form and send errors to browser. """
        if not form_obj.is_valid():
            for name, errors in form_obj.errors.items():
                if form_obj.prefix:
                    key = 'id_%s-%s' % (form_obj.prefix, name)
                else:
                    key = 'id_%s' % name
                self.form_data[key] = errors

    def replace(self, element, html):
        """ Replace the given DOM element with the given html. 
            The DOM element is specified using css identifiers.
            Some javascript libraries may have an extended syntax, 
            which can be used if you don't value portability.
        """
        self.replace_data[element] = html

    def hide(self, element):
        """ Hides the given DOM element.
            The DOM element is specified using css identifiers.
            Some javascript libraries may have an extended syntax, 
            which can be used if you don't value portability.
        """
        self.hide_data.append(element)

    def redirect(self, to, *args, **kwargs):
        """ Redirect the browser dynamically to another page. """
        if hasattr(to, 'get_absolute_url'):
            self.redirect_data = to.get_absolute_url()
            return
        try:
            self.redirect_data = urlresolvers.reverse(to, args=args, kwargs=kwargs)
            return
        except urlresolvers.NoReverseMatch:
            # If this is a callable, re-raise.
            if callable(to):
                raise
            # If this doesn't "feel" like a URL, re-raise.
            if '/' not in to and '.' not in to:
                raise
        
        # Finally, fall back and assume it's a URL
        self.redirect_data = to


    def extra(self, key, value):
        """ Send additional information to the browser. """
        self.extra_data[key] = value

    def render_to_response(self, template_name, dictionary=None, prefix=None, context_instance=None):
        """ Update any included templates. """
        # Because we have access to the request object, we can use request context
        # This is not analogous to render_to_strings interface
        if context_instance is None:
            context_instance = RequestContext(self.request)
        rendered_content = render_to_string(template_name, dictionary, context_instance=context_instance)
        dom_element = ".%s" % get_template_include_key(template_name, prefix)
        self.replace(dom_element, rendered_content)

    @property
    def json_response(self):
        """ Return a json response with our ajax data """
        elements = (
            ('extra', self.extra_data),
            ('messages', self.messages_data),
            ('forms', self.form_data),
            ('replace', self.replace_data),
            ('hide', self.hide_data),
            ('update', self.update_data),
            ('redirect', self.redirect_data),
            )
        return JsonResponse(dict((a,b) for a,b in elements if b))




from django.conf.urls.defaults import *

urlpatterns = patterns('',
    (r'^demo/', include('basic.urls')),
)

from django.conf import settings
if settings.DEBUG:
    urlpatterns += patterns('django.views.static',
        (r'^media/(?P<path>.*)$', 'serve', {'document_root': settings.MEDIA_ROOT}),
    )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages
setup(
    name = "Adjax",
    version = "1.0",
    packages = find_packages(exclude=["website*", "tests*", "design*"]),
    url = 'http://adjax.hardysoftware.com.au/',
    install_requires = ['django>=1.0'],
    author = "Will Hardy",
    author_email = "adjax@hardysoftware.com.au",
    description = "A framework for easing the development of Django sites with Ajax.",
    license = "New BSD",
    keywords = "ajax, django, framework",
    package_data = {
        'adjax': ['media/js/*.js'],
        }

)




# -*- coding: UTF-8 -*-

from django import template
from django.template.loader import get_template
from adjax.utils import get_key, get_template_include_key
from django.conf import settings


register = template.Library()


def adjax(parser, token):
    try:
        tag_name, object_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a single argument" % token.contents.split()[0]
    return DynamicValueNode(object_name)


class DynamicValueNode(template.Node):

    def __init__(self, object_name):
        self.object_name, self.field_name = object_name.rsplit(".", 1)
        self.instance = template.Variable(self.object_name)
        self.value = template.Variable(object_name)

    def render(self, context):
        instance = self.instance.resolve(context)
        if hasattr(instance, '_meta'):
            return '<span class="%s">%s</span>' % (get_key(instance, self.field_name), self.value.resolve(context))


def adjax_include(parser, token):
    bits = token.split_contents()
    try:
        tag_name, template_name = bits[:2]
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a template name" % bits[0]

    kwargs = {}
    for arg in bits[2:]:
        key, value = arg.split("=", 1)
        if key in ('prefix', 'wrapper'):
            kwargs[str(key)] = value
        else:
            raise template.TemplateSyntaxError, "invalid argument (%s) for %r tag" % (key, tag_name)

    return AdjaxIncludeNode(template_name, **kwargs)


class AdjaxIncludeNode(template.Node):

    def __init__(self, template_name, prefix=None, wrapper='"div"'):
        self.template_name = template.Variable(template_name)
        self.prefix = prefix and template.Variable(prefix) or None
        self.wrapper = template.Variable(wrapper)


    def render(self, context):
        template_name = self.template_name.resolve(context)
        wrapper = self.wrapper.resolve(context)
        prefix = self.prefix and self.prefix.resolve(context) or None
        key = get_template_include_key(template_name, prefix)
        try:
            content = get_template(template_name).render(context)
            return '<%s class="%s">%s</%s>' % (wrapper, key, content, wrapper)
        except template.TemplateSyntaxError, e:
            if settings.TEMPLATE_DEBUG:
                raise
            return ''
        except:
            return '' # Like Django, fail silently for invalid included templates.


# Register our tags
register.tag('adjax', adjax)
register.tag('adjax_include', adjax_include)

